@@ -3,10 +3,13 @@ package ingress
 import (
 	"context"
 	"fmt"
+	"regexp"
+	"strings"
 
 	iov1 "github.com/openshift/cluster-ingress-operator/pkg/api/v1"
 	logf "github.com/openshift/cluster-ingress-operator/pkg/log"
 	"github.com/openshift/cluster-ingress-operator/pkg/manifests"
+	retryable "github.com/openshift/cluster-ingress-operator/pkg/util/retryableerror"
 	"github.com/openshift/cluster-ingress-operator/pkg/util/slice"
 
 	appsv1 "k8s.io/api/apps/v1"
@@ -21,6 +24,7 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/types"
 	utilerrors "k8s.io/apimachinery/pkg/util/errors"
+	"k8s.io/apimachinery/pkg/util/sets"
 
 	"sigs.k8s.io/controller-runtime/pkg/cache"
 	"sigs.k8s.io/controller-runtime/pkg/client"
@@ -35,7 +39,14 @@ const (
 	controllerName = "ingress_controller"
 )
 
-var log = logf.Logger.WithName(controllerName)
+var (
+	log = logf.Logger.WithName(controllerName)
+	// tlsVersion13Ciphers is a list of TLS v1.3 cipher suites as specified by
+	// https://www.openssl.org/docs/man1.1.1/man1/ciphers.html
+	tlsVersion13Ciphers = sets.NewString("TLS_AES_128_GCM_SHA256", "TLS_AES_256_GCM_SHA384", "TLS_CHACHA20_POLY1305_SHA256",
+		"TLS_AES_128_CCM_SHA256", "TLS_AES_128_CCM_8_SHA256", "TLS_AES_128_GCM_SHA256", "TLS_AES_256_GCM_SHA384",
+		"TLS_CHACHA20_POLY1305_SHA256", "TLS_AES_128_CCM_SHA256", "TLS_AES_128_CCM_8_SHA256")
+)
 
 // New creates the ingress controller from configuration. This is the controller
 // that handles all the logic for implementing ingress based on
@@ -147,6 +158,10 @@ func (r *reconciler) Reconcile(request reconcile.Request) (reconcile.Result, err
 	}
 
 	// Only proceed if we can collect cluster config.
+	apiConfig := &configv1.APIServer{}
+	if err := r.client.Get(context.TODO(), types.NamespacedName{Name: "cluster"}, apiConfig); err != nil {
+		return reconcile.Result{}, fmt.Errorf("failed to get apiserver 'cluster': %v", err)
+	}
 	dnsConfig := &configv1.DNS{}
 	if err := r.client.Get(context.TODO(), types.NamespacedName{Name: "cluster"}, dnsConfig); err != nil {
 		return reconcile.Result{}, fmt.Errorf("failed to get dns 'cluster': %v", err)
@@ -159,10 +174,14 @@ func (r *reconciler) Reconcile(request reconcile.Request) (reconcile.Result, err
 	if err := r.client.Get(context.TODO(), types.NamespacedName{Name: "cluster"}, ingressConfig); err != nil {
 		return reconcile.Result{}, fmt.Errorf("failed to get ingress 'cluster': %v", err)
 	}
+	networkConfig := &configv1.Network{}
+	if err := r.client.Get(context.TODO(), types.NamespacedName{Name: "cluster"}, networkConfig); err != nil {
+		return reconcile.Result{}, fmt.Errorf("failed to get network 'cluster': %v", err)
+	}
 
 	// Admit if necessary. Don't process until admission succeeds. If admission is
 	// successful, immediately re-queue to refresh state.
-	if !isAdmitted(ingress) {
+	if !isAdmitted(ingress) || needsReadmission(ingress) {
 		if err := r.admit(ingress, ingressConfig, infraConfig); err != nil {
 			switch err := err.(type) {
 			case *admissionRejection:
@@ -178,10 +197,15 @@ func (r *reconciler) Reconcile(request reconcile.Request) (reconcile.Result, err
 	}
 
 	// The ingresscontroller is safe to process, so ensure it.
-	if err := r.ensureIngressController(ingress, dnsConfig, infraConfig, ingressConfig); err != nil {
-		return reconcile.Result{}, fmt.Errorf("failed to ensure ingresscontroller: %v", err)
+	if err := r.ensureIngressController(ingress, dnsConfig, infraConfig, ingressConfig, apiConfig, networkConfig); err != nil {
+		switch e := err.(type) {
+		case retryable.Error:
+			log.Error(e, "got retryable error; requeueing", "after", e.After())
+			return reconcile.Result{RequeueAfter: e.After()}, nil
+		default:
+			return reconcile.Result{}, err
+		}
 	}
-
 	return reconcile.Result{}, nil
 }
 
@@ -195,6 +219,10 @@ func (r *reconciler) admit(current *operatorv1.IngressController, ingressConfig
 	setDefaultDomain(updated, ingressConfig)
 	setDefaultPublishingStrategy(updated, infraConfig)
 
+	// The TLS security profile need not be defaulted.  If none is set, we
+	// get the default from the APIServer config (which is assumed to be
+	// valid).
+
 	if err := r.validate(updated); err != nil {
 		switch err := err.(type) {
 		case *admissionRejection:
@@ -204,6 +232,7 @@ func (r *reconciler) admit(current *operatorv1.IngressController, ingressConfig
 				Reason:  "Invalid",
 				Message: err.Reason,
 			})
+			updated.Status.ObservedGeneration = updated.Generation
 			if !ingressStatusesEqual(current.Status, updated.Status) {
 				if err := r.client.Status().Update(context.TODO(), updated); err != nil {
 					return fmt.Errorf("failed to update status: %v", err)
@@ -218,6 +247,7 @@ func (r *reconciler) admit(current *operatorv1.IngressController, ingressConfig
 		Status: operatorv1.ConditionTrue,
 		Reason: "Valid",
 	})
+	updated.Status.ObservedGeneration = updated.Generation
 	if !ingressStatusesEqual(current.Status, updated.Status) {
 		if err := r.client.Status().Update(context.TODO(), updated); err != nil {
 			return fmt.Errorf("failed to update status: %v", err)
@@ -235,6 +265,19 @@ func isAdmitted(ic *operatorv1.IngressController) bool {
 	return false
 }
 
+// needsReadmission returns a Boolean value indicating whether the given
+// ingresscontroller needs to be re-admitted.  Re-admission is necessary in
+// order to revalidate mutable fields that are subject to admission checks.  The
+// determination whether re-admission is needed is based on the
+// ingresscontroller's current generation and the observed generation recorded
+// in its status.
+func needsReadmission(ic *operatorv1.IngressController) bool {
+	if ic.Generation != ic.Status.ObservedGeneration {
+		return true
+	}
+	return false
+}
+
 func setDefaultDomain(ic *operatorv1.IngressController, ingressConfig *configv1.Ingress) bool {
 	var effectiveDomain string
 	switch {
@@ -285,6 +328,54 @@ func setDefaultPublishingStrategy(ic *operatorv1.IngressController, infraConfig
 	return false
 }
 
+// tlsProfileSpecForIngressController returns a TLS profile spec based on either
+// the profile specified by the given ingresscontroller, the profile specified
+// by the APIServer config if the ingresscontroller does not specify one, or the
+// "Intermediate" profile if neither the ingresscontroller nor the APIServer
+// config specifies one.  Note that the return value must not be mutated by the
+// caller; the caller must make a copy if it needs to mutate the value.
+func tlsProfileSpecForIngressController(ic *operatorv1.IngressController, apiConfig *configv1.APIServer) *configv1.TLSProfileSpec {
+	if hasTLSSecurityProfile(ic) {
+		return tlsProfileSpecForSecurityProfile(ic.Spec.TLSSecurityProfile)
+	}
+	return tlsProfileSpecForSecurityProfile(apiConfig.Spec.TLSSecurityProfile)
+}
+
+// hasTLSSecurityProfile checks whether the given ingresscontroller specifies a
+// TLS security profile.
+func hasTLSSecurityProfile(ic *operatorv1.IngressController) bool {
+	if ic.Spec.TLSSecurityProfile == nil {
+		return false
+	}
+	if len(ic.Spec.TLSSecurityProfile.Type) == 0 {
+		return false
+	}
+	return true
+}
+
+// tlsProfileSpecForSecurityProfile returns a TLS profile spec based on the
+// provided security profile, or the "Intermediate" profile if an unknown
+// or "Modern" security profile type is provided.  Note that the return value
+// must not be mutated by the caller; the caller must make a copy if it needs
+// to mutate the value.
+func tlsProfileSpecForSecurityProfile(profile *configv1.TLSSecurityProfile) *configv1.TLSProfileSpec {
+	if profile != nil {
+		if profile.Type == configv1.TLSProfileCustomType {
+			if profile.Custom != nil {
+				return &profile.Custom.TLSProfileSpec
+			}
+			return &configv1.TLSProfileSpec{}
+		} else if spec, ok := configv1.TLSProfiles[profile.Type]; ok {
+			// TODO remove when haproxy is built with an openssl version that supports tls v1.3.
+			if profile.Type == configv1.TLSProfileModernType {
+				return configv1.TLSProfiles[configv1.TLSProfileIntermediateType]
+			}
+			return spec
+		}
+	}
+	return configv1.TLSProfiles[configv1.TLSProfileIntermediateType]
+}
+
 // validate attempts to perform validation of the given ingresscontroller and
 // returns an error value, which will have a non-nil value of type
 // admissionRejection if the ingresscontroller is invalid, or a non-nil value of
@@ -303,6 +394,9 @@ func (r *reconciler) validate(ic *operatorv1.IngressController) error {
 	if err := validateDomainUniqueness(ic, ingresses.Items); err != nil {
 		errors = append(errors, err)
 	}
+	if err := validateTLSSecurityProfile(ic); err != nil {
+		errors = append(errors, err)
+	}
 
 	if err := utilerrors.NewAggregate(errors); err != nil {
 		return &admissionRejection{err.Error()}
@@ -334,46 +428,127 @@ func validateDomainUniqueness(desired *operatorv1.IngressController, existing []
 	return nil
 }
 
+var (
+	// validTLSVersions is all allowed values for TLSProtocolVersion.
+	validTLSVersions = map[configv1.TLSProtocolVersion]struct{}{
+		configv1.VersionTLS10: {},
+		configv1.VersionTLS11: {},
+		configv1.VersionTLS12: {},
+		// TODO: Add VersionTLS13 support after haproxy is built with an openssl
+		//  version that supports tls v1.3.
+	}
+
+	// isValidCipher is a regexp for strings that look like cipher names.
+	isValidCipher = regexp.MustCompile(`^[A-Za-z][A-Za-z0-9_+-]+$`).MatchString
+)
+
+// validateTLSSecurityProfile validates the given ingresscontroller's TLS
+// security profile, if it specifies one.
+func validateTLSSecurityProfile(ic *operatorv1.IngressController) error {
+	if !hasTLSSecurityProfile(ic) {
+		return nil
+	}
+
+	if ic.Spec.TLSSecurityProfile.Type != configv1.TLSProfileCustomType {
+		return nil
+	}
+
+	spec := ic.Spec.TLSSecurityProfile.Custom
+	if spec == nil {
+		return fmt.Errorf("security profile is not defined")
+	}
+
+	var errs []error
+
+	if len(spec.Ciphers) == 0 {
+		errs = append(errs, fmt.Errorf("security profile has an empty ciphers list"))
+	} else {
+		invalidCiphers := []string{}
+		for _, cipher := range spec.Ciphers {
+			if !isValidCipher(strings.TrimPrefix(cipher, "!")) {
+				invalidCiphers = append(invalidCiphers, cipher)
+			}
+		}
+		if len(invalidCiphers) != 0 {
+			errs = append(errs, fmt.Errorf("security profile has invalid ciphers: %s", strings.Join(invalidCiphers, ", ")))
+		}
+		filteredCiphers := filterTLS13Ciphers(spec.Ciphers)
+		if len(filteredCiphers) == 0 {
+			errs = append(errs, fmt.Errorf("security profile contains only tls v1.3 cipher suites"))
+		}
+	}
+
+	if _, ok := validTLSVersions[spec.MinTLSVersion]; !ok {
+		errs = append(errs, fmt.Errorf("security profile has invalid minimum security protocol version: %q", spec.MinTLSVersion))
+	}
+
+	return utilerrors.NewAggregate(errs)
+}
+
+// filterTLS13Ciphers filters any TLS v1.3 cipher suites from ciphers returning
+// a filtered list of cipher suites.
+func filterTLS13Ciphers(ciphers []string) []string {
+	filteredCiphers := []string{}
+	for i := 0; i < len(ciphers); i++ {
+		exist := false
+		if tlsVersion13Ciphers.Has(ciphers[i]) {
+			exist = true
+		}
+		if !exist {
+			filteredCiphers = append(filteredCiphers, ciphers[i])
+		}
+	}
+	return filteredCiphers
+}
+
 // ensureIngressDeleted tries to delete ingress, and if successful, will remove
 // the finalizer.
 func (r *reconciler) ensureIngressDeleted(ingress *operatorv1.IngressController) error {
-	if err := r.finalizeLoadBalancerService(ingress); err != nil {
-		return fmt.Errorf("failed to finalize load balancer service for %s: %v", ingress.Name, err)
+	errs := []error{}
+	if svcExists, err := r.finalizeLoadBalancerService(ingress); err != nil {
+		errs = append(errs, fmt.Errorf("failed to finalize load balancer service for ingress %s/%s: %v", ingress.Namespace, ingress.Name, err))
+	} else if svcExists {
+		errs = append(errs, fmt.Errorf("load balancer service exists for ingress %s/%s", ingress.Namespace, ingress.Name))
 	}
-	log.Info("finalized load balancer service for ingress", "namespace", ingress.Namespace, "name", ingress.Name)
 
 	// Delete the wildcard DNS record, and block ingresscontroller finalization
 	// until the dnsrecord has been finalized.
 	if err := r.deleteWildcardDNSRecord(ingress); err != nil {
-		return fmt.Errorf("failed to delete wildcard dnsrecord: %v", err)
+		errs = append(errs, fmt.Errorf("failed to delete wildcard dnsrecord for ingress %s/%s: %v", ingress.Namespace, ingress.Name, err))
 	}
 	if record, err := r.currentWildcardDNSRecord(ingress); err != nil {
-		return fmt.Errorf("failed to get current wildcard dnsrecord: %v", err)
-	} else {
-		if record != nil {
-			log.V(1).Info("waiting for wildcard dnsrecord to be deleted", "dnsrecord", record)
-			return nil
-		}
+		errs = append(errs, fmt.Errorf("failed to get current wildcard dnsrecord for ingress %s/%s: %v", ingress.Namespace, ingress.Name, err))
+	} else if record != nil {
+		errs = append(errs, fmt.Errorf("wildcard dnsrecord exists for ingress %s/%s", ingress.Namespace, ingress.Name))
 	}
 
 	if err := r.ensureRouterDeleted(ingress); err != nil {
-		return fmt.Errorf("failed to delete deployment for ingress %s: %v", ingress.Name, err)
+		errs = append(errs, fmt.Errorf("failed to delete deployment for ingress %s/%s: %v", ingress.Namespace, ingress.Name, err))
+	}
+	if deploy, err := r.currentRouterDeployment(ingress); err != nil {
+		errs = append(errs, fmt.Errorf("failed to get deployment for ingress %s/%s: %v", ingress.Namespace, ingress.Name, err))
+	} else if deploy != nil {
+		errs = append(errs, fmt.Errorf("deployment still exists for ingress %s/%s", ingress.Namespace, ingress.Name))
 	}
-	log.Info("deleted deployment for ingress", "namespace", ingress.Namespace, "name", ingress.Name)
 
-	// Clean up the finalizer to allow the ingresscontroller to be deleted.
-	if slice.ContainsString(ingress.Finalizers, manifests.IngressControllerFinalizer) {
-		updated := ingress.DeepCopy()
-		updated.Finalizers = slice.RemoveString(updated.Finalizers, manifests.IngressControllerFinalizer)
-		if err := r.client.Update(context.TODO(), updated); err != nil {
-			return fmt.Errorf("failed to remove finalizer from ingresscontroller %s: %v", ingress.Name, err)
+	if len(errs) == 0 {
+		// Remove the ingresscontroller finalizer.
+		if slice.ContainsString(ingress.Finalizers, manifests.IngressControllerFinalizer) {
+			updated := ingress.DeepCopy()
+			updated.Finalizers = slice.RemoveString(updated.Finalizers, manifests.IngressControllerFinalizer)
+			if err := r.client.Update(context.TODO(), updated); err != nil {
+				errs = append(errs, fmt.Errorf("failed to remove finalizer from ingresscontroller %s: %v", ingress.Name, err))
+			}
 		}
 	}
-	return nil
+	return utilerrors.NewAggregate(errs)
 }
 
-// ensureIngressController ensures all necessary router resources exist for a given ingresscontroller.
-func (r *reconciler) ensureIngressController(ci *operatorv1.IngressController, dnsConfig *configv1.DNS, infraConfig *configv1.Infrastructure, ingressConfig *configv1.Ingress) error {
+// ensureIngressController ensures all necessary router resources exist for a
+// given ingresscontroller.  Any error values are collected into either a
+// retryable.Error value, if any of the error values are retryable, or else an
+// Aggregate error value.
+func (r *reconciler) ensureIngressController(ci *operatorv1.IngressController, dnsConfig *configv1.DNS, infraConfig *configv1.Infrastructure, ingressConfig *configv1.Ingress, apiConfig *configv1.APIServer, networkConfig *configv1.Network) error {
 	// Before doing anything at all with the controller, ensure it has a finalizer
 	// so we can clean up later.
 	if !slice.ContainsString(ci.Finalizers, manifests.IngressControllerFinalizer) {
@@ -392,7 +567,7 @@ func (r *reconciler) ensureIngressController(ci *operatorv1.IngressController, d
 		return fmt.Errorf("failed to ensure namespace: %v", err)
 	}
 
-	deployment, err := r.ensureRouterDeployment(ci, infraConfig, ingressConfig)
+	deployment, err := r.ensureRouterDeployment(ci, infraConfig, ingressConfig, apiConfig, networkConfig)
 	if err != nil {
 		return fmt.Errorf("failed to ensure deployment: %v", err)
 	}
@@ -439,11 +614,9 @@ func (r *reconciler) ensureIngressController(ci *operatorv1.IngressController, d
 		errs = append(errs, fmt.Errorf("failed to list events in namespace %q: %v", "openshift-ingress", err))
 	}
 
-	if err := r.syncIngressControllerStatus(ci, deployment, lbService, operandEvents.Items, wildcardRecord, dnsConfig); err != nil {
-		errs = append(errs, fmt.Errorf("failed to sync ingresscontroller status: %v", err))
-	}
+	errs = append(errs, r.syncIngressControllerStatus(ci, deployment, lbService, operandEvents.Items, wildcardRecord, dnsConfig))
 
-	return utilerrors.NewAggregate(errs)
+	return retryable.NewMaybeRetryableAggregate(errs)
 }
 
 // IsStatusDomainSet checks whether status.domain of ingress is set.
