@@ -135,18 +135,18 @@ func (r *reconciler) currentLoadBalancerService(ci *operatorv1.IngressController
 	return service, nil
 }
 
-// finalizeLoadBalancerService removes finalizers from any LB service. This used
-// to be to help with DNS cleanup, but now that's no longer necessary, and so we
-// just need to clear the finalizer which might exist on existing resources.
-//
+// finalizeLoadBalancerService removes the "ingress.openshift.io/operator" finalizer
+// from the load balancer service of ci. This was for helping with DNS cleanup, but
+// that's no longer necessary. We just need to clear the finalizer which might exist
+// on existing resources.
 // TODO: How can we delete this code?
-func (r *reconciler) finalizeLoadBalancerService(ci *operatorv1.IngressController) error {
+func (r *reconciler) finalizeLoadBalancerService(ci *operatorv1.IngressController) (bool, error) {
 	service, err := r.currentLoadBalancerService(ci)
 	if err != nil {
-		return err
+		return false, err
 	}
 	if service == nil {
-		return nil
+		return false, nil
 	}
 	// Mutate a copy to avoid assuming we know where the current one came from
 	// (i.e. it could have been from a cache).
@@ -154,8 +154,10 @@ func (r *reconciler) finalizeLoadBalancerService(ci *operatorv1.IngressControlle
 	if slice.ContainsString(updated.Finalizers, manifests.LoadBalancerServiceFinalizer) {
 		updated.Finalizers = slice.RemoveString(updated.Finalizers, manifests.LoadBalancerServiceFinalizer)
 		if err := r.client.Update(context.TODO(), updated); err != nil {
-			return fmt.Errorf("failed to remove finalizer from service %s for ingress %s/%s: %v", service.Namespace, service.Name, ci.Name, err)
+			return true, fmt.Errorf("failed to remove finalizer from service %s/%s for ingress %s/%s: %v",
+				service.Namespace, service.Name, ci.Namespace, ci.Name, err)
 		}
 	}
-	return nil
+	log.Info("finalized load balancer service for ingress", "namespace", ci.Namespace, "name", ci.Name)
+	return true, nil
 }
