@@ -16,6 +16,8 @@ import (
 	"testing"
 	"time"
 
+	"gopkg.in/yaml.v2"
+
 	configv1 "github.com/openshift/api/config/v1"
 	operatorv1 "github.com/openshift/api/operator/v1"
 
@@ -36,6 +38,7 @@ import (
 	metav1 "k8s.io/apimachinery/pkg/apis/meta/v1"
 	"k8s.io/apimachinery/pkg/runtime/schema"
 	"k8s.io/apimachinery/pkg/types"
+	"k8s.io/apimachinery/pkg/util/sets"
 	"k8s.io/apimachinery/pkg/util/wait"
 
 	"k8s.io/apiserver/pkg/storage/names"
@@ -306,6 +309,11 @@ func TestIngressControllerScale(t *testing.T) {
 		t.Fatalf("router deployment has invalid spec.selector: %v", err)
 	}
 
+	oldRsList := &appsv1.ReplicaSetList{}
+	if err := kclient.List(context.TODO(), oldRsList, client.MatchingLabelsSelector{selector}); err != nil {
+		t.Fatalf("failed to list replicasets for ingresscontroller: %v", err)
+	}
+
 	resource := schema.GroupResource{
 		Group:    "operator.openshift.io",
 		Resource: "ingresscontrollers",
@@ -367,6 +375,24 @@ func TestIngressControllerScale(t *testing.T) {
 	if err := waitForIngressControllerCondition(kclient, 5*time.Minute, defaultName, defaultAvailableConditions...); err != nil {
 		t.Fatalf("failed to observe expected conditions: %v", err)
 	}
+
+	// Ensure the deployment did not create a new replicaset
+	// (see <https://bugzilla.redhat.com/show_bug.cgi?id=1783007>).
+	newRsList := &appsv1.ReplicaSetList{}
+	if err := kclient.List(context.TODO(), newRsList, client.MatchingLabelsSelector{selector}); err != nil {
+		t.Fatalf("failed to list replicasets for ingresscontroller: %v", err)
+	}
+	oldRsIds := sets.String{}
+	for _, rs := range oldRsList.Items {
+		oldRsIds.Insert(string(rs.UID))
+	}
+	newRsIds := sets.String{}
+	for _, rs := range newRsList.Items {
+		newRsIds.Insert(string(rs.UID))
+	}
+	if !oldRsIds.IsSuperset(newRsIds) {
+		t.Fatalf("scaling the deployment created a new replicaset\nold replicaset list:\n%#v\nnew replicaset list:\n%#v)", oldRsList.Items, newRsList.Items)
+	}
 }
 
 func getScaleClient() (scale.ScalesGetter, error) {
@@ -465,9 +491,9 @@ func TestRouterCACertificate(t *testing.T) {
 	}
 }
 
-// TestPodDisruptionBudgetExists verifies that a PodDisruptionBudget resource
-// exists for the default ingresscontroller.
-func TestPodDisruptionBudgetExists(t *testing.T) {
+// TestPodDisruptionBudgetDoesNotExist verifies that a PodDisruptionBudget
+// resource does not exist for the default ingresscontroller.
+func TestPodDisruptionBudgetDoesNotExist(t *testing.T) {
 	ic := &operatorv1.IngressController{}
 	if err := kclient.Get(context.TODO(), defaultName, ic); err != nil {
 		t.Fatalf("failed to get default ingresscontroller: %v", err)
@@ -479,7 +505,10 @@ func TestPodDisruptionBudgetExists(t *testing.T) {
 
 	pdb := &policyv1beta1.PodDisruptionBudget{}
 	if err := kclient.Get(context.TODO(), controller.RouterPodDisruptionBudgetName(ic), pdb); err != nil {
-		t.Fatalf("failed to get default ingresscontroller poddisruptionbudget: %v", err)
+		if errors.IsNotFound(err) {
+			return
+		}
+		t.Errorf("error getting default ingresscontroller poddisruptionbudget: %v", err)
 	}
 }
 
@@ -552,6 +581,69 @@ func TestInternalLoadBalancer(t *testing.T) {
 	}
 }
 
+// TestTLSSecurityProfile creates an ingresscontroller with no explicit TLS
+// profile, then verifies that the operator sets the default "Intermediate" TLS
+// profile, then updates the ingresscontroller to use a custom TLS profile, and
+// then verifies that the operator reflects the custom profile in its status.
+func TestTLSSecurityProfile(t *testing.T) {
+	name := types.NamespacedName{Namespace: operatorNamespace, Name: "test"}
+	domain := name.Name + "." + dnsConfig.Spec.BaseDomain
+	ic := newPrivateController(name, domain)
+	if err := kclient.Create(context.TODO(), ic); err != nil {
+		t.Fatalf("failed to create ingresscontroller %s: %v", name, err)
+	}
+	defer assertIngressControllerDeleted(t, kclient, ic)
+	conditions := []operatorv1.OperatorCondition{
+		{Type: operatorv1.IngressControllerAvailableConditionType, Status: operatorv1.ConditionTrue},
+		{Type: operatorv1.LoadBalancerManagedIngressConditionType, Status: operatorv1.ConditionFalse},
+		{Type: operatorv1.DNSManagedIngressConditionType, Status: operatorv1.ConditionFalse},
+	}
+	if err := waitForIngressControllerCondition(kclient, 5*time.Minute, name, conditions...); err != nil {
+		t.Fatalf("failed to observe expected conditions: %v", err)
+	}
+
+	if err := kclient.Get(context.TODO(), name, ic); err != nil {
+		t.Fatalf("failed to get ingresscontroller %s: %v", name, err)
+	}
+	if ic.Status.TLSProfile == nil {
+		t.Fatalf("ingresscontroller status has no security profile")
+	}
+	intermediateProfileSpec := configv1.TLSProfiles[configv1.TLSProfileIntermediateType]
+	if !reflect.DeepEqual(*ic.Status.TLSProfile, *intermediateProfileSpec) {
+		expected, _ := yaml.Marshal(intermediateProfileSpec)
+		actual, _ := yaml.Marshal(*ic.Status.TLSProfile)
+		t.Fatalf("ingresscontroller status has unexpected security profile spec.\nexpected:\n%s\ngot:\n%s", expected, actual)
+	}
+
+	customProfileSpec := configv1.TLSProfileSpec{
+		Ciphers:       []string{"ECDHE-ECDSA-AES256-GCM-SHA384"},
+		MinTLSVersion: configv1.VersionTLS12,
+	}
+	ic.Spec.TLSSecurityProfile = &configv1.TLSSecurityProfile{
+		Type: configv1.TLSProfileCustomType,
+		Custom: &configv1.CustomTLSProfile{
+			TLSProfileSpec: customProfileSpec,
+		},
+	}
+	if err := kclient.Update(context.TODO(), ic); err != nil {
+		t.Errorf("failed to update ingresscontroller %s: %v", name, err)
+	}
+	err := wait.PollImmediate(1*time.Second, 5*time.Minute, func() (bool, error) {
+		if err := kclient.Get(context.TODO(), name, ic); err != nil {
+			t.Fatalf("failed to get ingresscontroller %s: %v", name, err)
+		}
+		if !reflect.DeepEqual(*ic.Status.TLSProfile, customProfileSpec) {
+			return false, nil
+		}
+		return true, nil
+	})
+	if err != nil {
+		expected, _ := yaml.Marshal(customProfileSpec)
+		actual, _ := yaml.Marshal(*ic.Status.TLSProfile)
+		t.Fatalf("ingresscontroller status has unexpected security profile spec.\nexpected:\n%s\ngot:\n%s", expected, actual)
+	}
+}
+
 func newLoadBalancerController(name types.NamespacedName, domain string) *operatorv1.IngressController {
 	repl := int32(1)
 	return &operatorv1.IngressController{
